#pragma once

#include "List.h"

#include <vector>

class ListFactory;

template<typename T>
class ListT : public List
{
public:
	ListT() = delete;

	ListT(List::Type type, std::size_t size, T value)
		: mType(type)
	{
		if (size > 0)
		{
			initialize(size, value);
		}
	}

	virtual Type getType() const override
	{
		return mType;
	}

	const T* getDataPtr() const
	{
		return mData.data();
	}

	const std::vector<T>& getData() const
	{
		return mData;
	}

	void setData(const std::vector<T>& data)
	{
		mData = data;
	}

	void setDataItem(std::size_t index, T item)
	{
		if (index < mData.size())
		{
			mData[index] = item;
		}
	}

	void initialize(std::size_t size, T value)
	{
		mData.resize(size);
		for (std::size_t idx = 0; idx < size; idx++)
		{
			mData[idx] = value;
		}
	}

	std::size_t getLength() const override
	{
		return mData.size();
	}

private:
	List::Type mType{ List::Type::UNKNOWN };
	std::vector<T> mData;
};