#pragma once

#include "Bounds.h"
#include "List.h"
#include "ListT.h"

#include <vector>
#include <memory>
#include <stdexcept>

class AbstractGrid
{
public:
    AbstractGrid(const Bounds& bounds, std::size_t numPointsX, std::size_t numPointsY, List::Type dataType = List::Type::DOUBLE);

    const Bounds& getBounds() const;

    double getXSpacing() const;

    double getYSpacing() const;

    List::Type getDataType();

    template<typename T>
    const std::vector<T> getData() const
    {
        if (!dynamic_cast<ListT<T>*>(mData.get()))
        {
            throw std::logic_error("Invalid data type request for Grid data.");
        }

        return dynamic_cast<ListT<T>*>(mData.get())->getData();

    }

    template<typename T>
    void setData(const std::vector<std::size_t> indices, const std::vector<T>& values)
    {
        if (auto list_t = dynamic_cast<ListT<T>*>(mData.get()))
        {
            list_t->setData(indices, values);
        }
    }

    virtual void initializeData(List::Type dataType) = 0;

    void resetBounds(const Bounds& bounds);

protected:
    Bounds mBounds;
    std::unique_ptr<List> mData;
    std::size_t mNumX{ 0 };
    std::size_t mNumY{ 0 };
};